using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using Win32 = Microsoft.Win32;

namespace VistaMidiFix {
	public partial class MainForm : Form {

		public MainForm() {
			InitializeComponent();
			this.Text = Application.ProductName;
			int x = DefaultMidiDevice;
		}

		// List population:
		private void MainForm_Load(object sender, EventArgs e) {

			// Populate the list of MIDI devices:
			ushort DeviceCount = WinMM.Midi.midiOutGetNumDevs();

			// Check there's a sane number:
			if (DeviceCount < 1) {
				MessageBox.Show(this, "No MIDI devices found!", null, MessageBoxButtons.OK, MessageBoxIcon.Error);
				this.Close();
				return;
			}


			// Populate the box:

			int DefaultId = -1;
			try {
				DefaultId = this.DefaultMidiDevice;
			} catch (Exception ex) {
				MessageBox.Show(this, "Error reading default MIDI output device ID: " + ex.Message, null, MessageBoxButtons.OK, MessageBoxIcon.Error);
			}

			for (int i = 0; i < DeviceCount; ++i) {
				WinMM.Midi.MIDIOUTCAPS Caps = default(WinMM.Midi.MIDIOUTCAPS);
				WinMM.Midi.Error Error = WinMM.Midi.midiOutGetDevCaps(i, ref Caps, Marshal.SizeOf(Caps));
				if (Error == WinMM.Midi.Error.MMSYSERR_NOERROR) {
					this.MidiOutDevicePicker.Items.Add(Caps.szPname);
				} else {
					this.MidiOutDevicePicker.Items.Add("Device #" + i.ToString() + ": " + Error.ToString());
				}
			}
			if (DefaultId < this.MidiOutDevicePicker.Items.Count && DefaultId > -2) this.MidiOutDevicePicker.SelectedIndex = DefaultId;
		}

		#region Button Response

		// "OK"
		private void ConfirmOK_Click(object sender, EventArgs e) {
			try {
				this.DefaultMidiDevice = this.MidiOutDevicePicker.SelectedIndex;
				this.Close();
			} catch (Exception ex) {
				MessageBox.Show(this, "Error changing default MIDI output device ID: " + ex.Message, null, MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		// "Cancel"
		private void ConfirmCancel_Click(object sender, EventArgs e) {
			this.Close();
		}

		#endregion

		#region Win32 Registry

		private const string MidiDefaultDeviceKey = @"Software\Microsoft\ActiveMovie\devenum\{4EFE2452-168A-11D1-BC76-00C04FB9453B}\Default MidiOut Device";

		/// <summary>Gets or sets the default MIDI device number.</summary>
		private int DefaultMidiDevice {
			get {
				Win32.RegistryKey Key = null;
				try {
					Key = Win32.Registry.CurrentUser.OpenSubKey(MidiDefaultDeviceKey);
					return (int)Key.GetValue("MidiOutId");
				} finally {
					if (Key != null) Key.Close();
				}

			}
			set {
				Win32.RegistryKey Key = null;
				try {
					Key = Win32.Registry.CurrentUser.OpenSubKey(MidiDefaultDeviceKey, Win32.RegistryKeyPermissionCheck.ReadWriteSubTree, System.Security.AccessControl.RegistryRights.SetValue);
					Key.SetValue("MidiOutId", value);
				} finally {
					if (Key != null) Key.Close();
				}
			}
		}

		#endregion

	}
}